/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar;

import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.xml.resolver.tools.CatalogResolver;
import org.sourceforge.xradar.logging.LogUtils;
import org.sourceforge.xradar.resolver.XRadarCatalogResolver;
import org.sourceforge.xradar.resolver.XRadarStreamHandlerFactory;
import org.sourceforge.xradar.statics.Param;
import org.sourceforge.xradar.statics.Preambule;
import org.sourceforge.xradar.util.FileUtils;
import org.sourceforge.xradar.util.StreamUtils;
import org.sourceforge.xradar.util.URLUtils;

/**
 *
 * This class regroups all the methods and property common to statics and dynamics
 * process. As it has not yet any abtract methods, it may be refactor to a simple
 * Utility classe at the end...
 *
 * @author rpelisse, belaran@gmail.com
 *
 */
public class AbstractProcess
{
	private static final Logger logger = Logger.getLogger(AbstractProcess.class.getSimpleName());
	protected XSLTMerger merger = new XSLTMerger();

	public static final	String XRADAR_CONFIG_FILE = "radar-config-file";

	//public static final String SYSTEM_FILE_SEPARATOR = File.separator;
	// According to XRadarUtil.normalizeFileName that replace \ by /
	public static final String SYSTEM_FILE_SEPARATOR = "/";
	
	
	protected static final String RELEASE_DATE = "date";
	protected static final String RELEASE_SYSTEM = "system";
	protected static final String RELEASE_VERSION = "version";

	protected static final String SYSTEM_QUALITY_REPORT = "system_quality_report.xml";
	protected static final String NORMALIZED_REPORT_NAME = "radar_report_normalized.xml";
	protected static final String MASTER_REPORT_NAME = "masterreport_final.xml";
	protected static final String LOG_STATUS = "log";

	protected static final String DOCS_HOME = "docs-home";

	private String docsHome;
	private String config;
	private boolean debug = true;

	public final static String FILE_PROTOCOL = "file:/";
	public final static String XRADAR_PROTOCOL = "xradar:/";

	protected AbstractProcess() {
		// This class is not really abstract as none of its children have 
		//  share common signature , anyway,  it should not be instantiated, 
		//therefore, the constructor is protected.</p>
	}

	public void initializedXmlCatalog(boolean useCatalog) {
	    if ( useCatalog ) {
			logger.log(Level.WARNING,"Offline mode, using inner XML Catalog from path");
			CatalogResolver catalog = new XRadarCatalogResolver();
			XSLTMerger.setEntityResolver(catalog);
			XSLTMerger.setUriResolver(catalog);
	    }		
	}
	
	
	/**
	 * @return the config
	 */
	public String getConfig() {
		return config;
	}

	/**
	 * @param config the config to set
	 */
	public void setConfig(String config) {
		this.config = config;
	}

	/**
	 * @return the debug
	 */
	public boolean isDebug() {
		return debug;
	}

	/**
	 * @param debug the debug to set
	 */
	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	/**
	 * @return the docsHome
	 */
	public String getDocsHome() {
		return docsHome;
	}

	/**
	 * @param docsHome the docsHome to set
	 * @throws XRadarException if the file does not exist or does  not have the write permissions
	 */
	public void setDocsHome(String docsHome) throws XRadarException {
	    // If running on windows, docs-home maybe a perverted URL containing base \ and /
	    // this.docsHome = XRadarUtils.normalizeFileName(docsHome);
	    this.docsHome = docsHome;

		//TODO: Add some test ( dir exits ? writable ?)
		//File dir = new File (docsHome)
		 //if (dir.exists() && dir.canWrite()){
		logger.finest("DOCS_HOME is :" + this.docsHome);
		//else 
		//throw new XRadarException ("File " + docsHome + " does not exist or is not writable");*/
	}

	
	/*
	 * Simple wrapper to increase code readability
	 */
	protected String getDefault(String id)
	{
		return DefaultReportValues.getString(id);
	}

	/*
	 * Simple wrapper to increase code readability
	 */
	protected int getDefaultIntValue(String id)
	{
		return Integer.parseInt(DefaultReportValues.getString(id));
	}

	/*
	 * Set the $log param according to the current log level
	 */
	protected void addLogLevelDefinitionToPreambule(Preambule preambule)
	{
		preambule.getParams().put(
				LOG_STATUS,
				new Param(LOG_STATUS,Boolean.valueOf(LogUtils.isLogEnabled(logger)).toString()));
	}

	protected void copyImagesToWebsite(String imagesTargetPrefix) throws XRadarException
	{
		logger.fine("Copying images to website");
		int nbStaticResources = Integer.valueOf(DefaultReportValues.getString("radar.internal.static.nbResources"));
		logger.finest("Copying" + nbStaticResources + " statics resources.");
		for ( int resource = 1; resource <= nbStaticResources ; resource++ )
		{
			logger.finest("Loading resources url and target");
			String resourcePath = DefaultReportValues.getString("radar.internal.static.resources." + resource);
			String targetPath = DefaultReportValues.getString("radar.internal.static.resources.target." + resource);
			logger.finest("Static resource ID:[" + resource + "]. Copying it from " + resourcePath + " to " + imagesTargetPrefix + SYSTEM_FILE_SEPARATOR + targetPath );
			String filename = imagesTargetPrefix + SYSTEM_FILE_SEPARATOR + targetPath;
			FileUtils.createParentDirectories(filename);
			StreamUtils.copy(URLUtils.openStream(resourcePath),StreamUtils.createOutputStream(filename));
		}
		logger.fine("Images resources copying done.");
	}

	
	/**
	 *  <p>Very important method ! This method register the resolver, which allow the JVM to 
	 *  actual resolve the 'xradar' protocol and find the file requested.</p>
	 */
	public static void registerResolvers() {
		/*
		 * Here there is an issue. if running this several time in the same jvm
		 * the StreamHandler is already defined and the following exception is
		 * thrown. i did not find a better way to avoid this than doing this catch.
		 */
		// FIXME: This exception catching is ugly !!!!!!
	    	// TODO: Maybe simply use the setURIResolver on the transformer ?
		try
		{
			// Adding to JVM our own URL resolver to resolve the 'xradar' protocol
			URL.setURLStreamHandlerFactory(new XRadarStreamHandlerFactory());
		}
		catch ( java.lang.Error e)
		{
			//FIXME: this so ugly, it actually requires an other fixme !
			// Only if we run into this error we should not take care of the exception
			if ( ! "class java.lang.Error".equals(e.getClass()) &&
				 ! "factory already defined".equals(e.getMessage() )	)
			{
				throw e;
			}
		}
	}
}
