/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.sourceforge.xradar.config.ReleaseData;
import org.sourceforge.xradar.config.ReleaseDataLoader;
import org.sourceforge.xradar.statics.MergeProduct;
import org.sourceforge.xradar.statics.Param;
import org.sourceforge.xradar.statics.Report;
import org.sourceforge.xradar.statics.Statics;
import org.sourceforge.xradar.statics.renderers.XslHtmlWebsite;

/**
 * 
 * @author Romain PELISSE, belaran@gmail.com
 * @author Zahir Nuckchady, zahir.nu@gmail.com
 */
public class XRadarStaticsCmdLine {

	/**
	 * Command line to use xradar:
	 * $ java -cp org.sourceforge.xradar.Statics --pmd pmd.xml --jpdepend ... --target --offline --config etc/radar-config
	 *
	 * @param args
	 * @throws XRadarException
	 */
	public static void runXRadarStatics(String[] args) throws XRadarException {
		// Dealing with the arguments
		List<Report> reports = new ArrayList<Report>();
		XRadarParameters parameters = XRadarStaticsCmdLine.extractArguments(reports,args);

		// Is target a directory and  an absolute path ? Target MUST BE an absolute path...
		parameters.setTarget(new File(parameters.getTarget()).getAbsolutePath());

		//	Settings...
		Param radarConfig = new Param(Statics.XRADAR_CONFIG_FILE,parameters.getXradarConfig());
		ReleaseData releaseData = new ReleaseDataLoader("testproject/etc/release1.properties").load();

		XslHtmlWebsite staticEngine = new XslHtmlWebsite();
		// If Offline is specified in the arguments, the xml catalog will be used
		staticEngine.initializedXmlCatalog(parameters.isOffline());
		staticEngine.setDocsHome(parameters.getTarget());
		staticEngine.setDebug(parameters.isDebug());
		staticEngine.setClassesDirectory(parameters.getClassesDirectory());

		// Add reports to the statics instance	
		for ( Report report : reports ) {
			staticEngine.addReport(report);
		}

		// Executing the merging and then the PostProcessing
		Map<String,Param> params = new HashMap<String,Param>();
		params.put(Statics.JAVADOC_ENABLED,new Param(Statics.JAVADOC_ENABLED,String.valueOf(parameters.isJavadocEnabled()))); 
		params.put(Statics.JAVADOC_ROOT,new Param(Statics.JAVADOC_ROOT,parameters.getJavadocRoot()));
		params.put(Statics.JAVA2HTML_ROOT,new Param(Statics.JAVA2HTML_ROOT,parameters.getJava2htmlRoot()));
		params.put(Statics.XRADAR_CONFIG_FILE,radarConfig);
		// Merge and the post process
		MergeProduct products = staticEngine.executePostProcessing(staticEngine.executeMerge(),releaseData,radarConfig);
		staticEngine.generateHtmlReport(products, params);
	}


	/**
	 * <p>Checks if all the necessary arguments required to generate the XRadar
	 * report are present.</p>
	 * @param list, contains all the arguments in the command line
	 * @throws IllegalArgumentException if a required argument is missing
	 **/
	private static void checkCompulsoryArgs(List<String> list) {
		String [] compulsoryArgs = new String [] {"--target","--properties","--xradarConfig"};
		//String [] classesGiven = new String [] {"--classes","--target","--properties","--xradarConfig"};
		for(String argument :compulsoryArgs){
			if( ! list.contains(argument))
				throw new IllegalArgumentException(argument + " is a compulsory argument in order to generate the XRadar report");
		}
	}

	private static void checkJdependClasses(List<String> list) {
		boolean classes = false, jdepend = false;
		Iterator <String> iterator = list.iterator();
		while ( iterator.hasNext() ) {
			String arg = iterator.next();
			if (arg.contains("--classes")){
				classes = true;
			}
			if ( arg.contains("--jdepend")){
				jdepend = true;
			}
		}
		if (!( classes || jdepend ) ){
				throw new IllegalArgumentException("If you do not have a jdepend report already generated, XRadar can run JDepend for you if you provide it the directory where your project directory is.");
			}
		}

	/**
	 * <p>Extract all the arguments provided in the command line.
	 * Before generating the reports, we verify if the arguments and the files exist.</p>
	 * @param report, which contains all the arguments provided
	 * @param args, which represents the arguments provided in the command line
	 * @throws IllegalArgumentException when the file specified does not exist, a wrong file is specified or the command does not exist
	 **/
	private static XRadarParameters extractArguments(List<Report> reports,String[] args) {

		List<String> list = Arrays.asList( args );
		XRadarParameters xradarParameters = new XRadarParameters();
		checkCompulsoryArgs(list);
		checkJdependClasses(list);
		Iterator <String> iterator = list.iterator();
		List<CommandLineArgument> cmdArguments = createArgsReferences() ;		

		while ( iterator.hasNext() ) {
			String arg = iterator.next();

			if ( !arg.startsWith("--") ) { // If no command is specified (--pmd, --jdepend), reports cannot be generated
				throw new IllegalArgumentException(arg+ " is not a valid command. \nCommand should always start with --");
			}
			// Test if the arguments exist. E.x: pmd, checkstyle ...
			// FIXME: this is ugly !
			CommandLineArgument cmd = null;
			for (CommandLineArgument tempCmd : cmdArguments ) {
				if ( tempCmd.getArgument().equals(arg) ) {
					cmd = tempCmd;
				}
			}
			// If the command does not exist, throws an exception
			if (cmd == null)
				throw new IllegalArgumentException("The " +arg+ " command is unknown to Xradar");
			// Command found, test if value for argument is provided
			if ( cmd.getNbValue() == 0 ) {
				if ( arg.contains("offline") ){
					xradarParameters.setOffline(true);								
				}
				if ( arg.contains("online") ){
					xradarParameters.setOffline(false);								
				}
				if ( arg.contains("debug") ){
					xradarParameters.setDebug(true);								
				}
				if ( arg.contains("javadocEnabled") ){
					xradarParameters.setJavadocEnabled(true);								
				}
			}
			else if ( (cmd.getNbValue() == 1) && (iterator.hasNext()) ) {
				String value = iterator.next();
				// If value for the argument is absent, throws an Exception						
				if ( value.startsWith("--") ) {
					throw new IllegalArgumentException ("Missing " + arg + " value for " + arg +" command");
				}
				// Verify if the value is a file path and if the file exists
				fileExists(value);
				if ( cmd.isValueAFile(0) ) {
					File file = new File(value);
					if ( ! file.exists() && 
						(  "--target".equals(arg) && ! file.mkdir() ) ) { // No target directory ? let's try to create it
						throw new IllegalArgumentException("File " + value + " does not exist.");
					}
				}
				if ( arg.contains("javadocRoot") ){
					xradarParameters.setJavadocRoot(value);								
				}
				else  if ( arg.contains("java2htmlRoot") ){
					xradarParameters.setJava2htmlRoot(value);								
				}
				else if ( arg.contains("properties") ){
					xradarParameters.setProperties(value);								
				}
				else if ( arg.contains("xradarConfig") ){
					xradarParameters.setXradarConfig(value);								
				}
				else if ( arg.contains("target") ){
					xradarParameters.setTarget(value);								
				} 
				else if ( arg.contains("classes") ){
					xradarParameters.setClassesDirectory(value);								
				} 
				else {// Create report if arguments are valid
					reports.add(new Report(arg.substring(2, arg.length()),value));
				}
			}
			else {
				throw new IllegalArgumentException ("Missing value for argument " + arg + "\nRight statement should be: \"" + arg + " value\"");
			}
		}			
		return xradarParameters;
	}

	/**
	 *Verifies if the file exists
	 *@param fileName and Command
	 */
	private static void fileExists (String fileName) {
		File file = new File(fileName);
		if ( ! file.exists()){
			throw new IllegalArgumentException("File " + file + " does not exist");
		}
	}

	/**
	 * <p>Creates a List with all the possible arguments provided in the command line.</p>
	 **/
	private static List<CommandLineArgument> createArgsReferences() {
		List<CommandLineArgument> cmdArguments = new ArrayList<CommandLineArgument>();
		cmdArguments.add(new CommandLineArgument("--pmd",1,true));
		cmdArguments.add(new CommandLineArgument("--checkstyle",1,true));
		cmdArguments.add(new CommandLineArgument("--jdepend",1,true));
		cmdArguments.add(new CommandLineArgument("--cpd",1,true));
		cmdArguments.add(new CommandLineArgument("--ckjm",1,true));
		cmdArguments.add(new CommandLineArgument("--javancss",1,true));
		cmdArguments.add(new CommandLineArgument("--findbugs",1,true));
		cmdArguments.add(new CommandLineArgument("--target",1,true)); 
		cmdArguments.add(new CommandLineArgument("--properties",1,true));
		cmdArguments.add(new CommandLineArgument("--xradarConfig",1,true));
		cmdArguments.add(new CommandLineArgument("--classes",1,true));
		cmdArguments.add(new CommandLineArgument("--javadocRoot",1,true));
		cmdArguments.add(new CommandLineArgument("--java2htmlRoot",1,true));
		cmdArguments.add(new CommandLineArgument("--javadocEnabled",0,false));
		cmdArguments.add(new CommandLineArgument("--offline",0,false));
		cmdArguments.add(new CommandLineArgument("--online",0,false));
		cmdArguments.add(new CommandLineArgument("--debug",0,false));

		return cmdArguments;
	}
}
