/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar;

import java.io.IOException;
import java.io.InputStream;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.URIResolver;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamSource;

import org.sourceforge.xradar.statics.Param;
import org.sourceforge.xradar.statics.Preambule;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author Romain PELISSE, belaran@gmail.com
 *
 */
public class XSLTMerger {

	private final static Logger logger = Logger.getLogger(XSLTMerger.class.getSimpleName());;
	
   	private static EntityResolver entityResolver;
   	private static URIResolver uriResolver;
   	
	private Transformer transformer;
	private InputStream currentXslt;
	private InputSource currentXmlFile;
	private Preambule preambule;
	private Result xmlResult;

	/**
	 * Default constructor:
	 * Warning, this constructor register the xradar protocol handler !
	 */
	public XSLTMerger()
	{
		AbstractProcess.registerResolvers();
	}

	/*
	 * Prepare factory and get new transformer
	 */
	private void prepareFactory() throws TransformerConfigurationException
	{
		TransformerFactory factory = TransformerFactory.newInstance();
		StreamSource src = new StreamSource(this.currentXslt);
		this.transformer = factory.newTransformer(src);
	}

	/*
	 * Load the xml file and return it.
	 * Uses XML Catalog to resolve entity if provided.
	 */
	private DOMSource loadXMLFile() throws ParserConfigurationException, SAXException, IOException
	{
		// Loading XML file
		DocumentBuilder parser = DocumentBuilderFactory.newInstance().newDocumentBuilder();
		// XRadar come with it own xml catalog in order to NOT download third party dtds
		if ( XSLTMerger.entityResolver != null )
		    parser.setEntityResolver(XSLTMerger.entityResolver);
		Document document =  parser.parse(this.currentXmlFile);
		return new DOMSource(document);
	}

	
	private final static String JAVA_VERSION =  System.getProperty("java.version");
	private final static String ANT_HOME = System.getProperty("ant.home") ;
	/**
	 * <p>FIXME: For mysterious reasons, XML Catalog has very bad side effect when
	 * run inside Ant and the Java5, therefore this ugly hack fix the issue.</p>
	 * @return
	 */
	private boolean canUseXmlCatalog() {
		if ( ANT_HOME != null && JAVA_VERSION.startsWith("1.5") )
			return false;
		return true;
	}
	
	/*
	 * Execute the merge, with the provided XML Catalog (if there is one).
	 * 	
	 */
	private Result merge() throws ParserConfigurationException, SAXException, IOException, TransformerException
	{
		// Preparation work
		this.prepareFactory();
		// Loading xml file
		logger.log(Level.FINEST,"Processing XML file:" + this.currentXmlFile);
		// FIXME: using DOMSource here may (again) lead to performance issue on large project
		DOMSource xml = this.loadXMLFile();
		// Preparing transformation
		this.preparePreambule();
		// Passing Catalog to transformer to avoid any internet lookup for dtd
		if ( XSLTMerger.uriResolver != null && canUseXmlCatalog() )
		    this.transformer.setURIResolver(XSLTMerger.uriResolver); 
		// Execute the transformation
		this.transformer.transform(xml, xmlResult);
		// Return the result
		return xmlResult;
	}

	/*
	 * Deals with any operation to do before executing the transformation
	 * (most likely the passing of parameters to the xslt file)
	 */
	private void preparePreambule()
	{
		// do nothing if no preambule is defined or if it is empty
		if ( this.preambule != null &&
			 this.preambule.getParams() != null &&
			 ! this.preambule.getParams().isEmpty() )
		{
			// First we clean the object of any previous parameters
			this.transformer.clearParameters();
			// We add each param described in the preambule
			for ( Param param : this.preambule.getParams().values() )
			{
				if ( param.getName() != null && ! "".equals(param.getName()) &&
					 param.getExpression() != null && ! "".equals(param.getExpression()))
					this.transformer.setParameter(param.getName(),param.getExpression());
			}
		}
	}

	/**
	 * Transform the currentXmlFile according to the currentXslt file.
	 *
	 * @param currentXmlFile
	 * @param currentXslt
	 * @return
	 */

	public Result merge(InputSource currentXmlFile, InputStream currentXslt) {
		this.currentXslt = currentXslt;
		this.currentXmlFile = currentXmlFile;
		Result result = null;
		try {
			result = this.merge();
		} catch (ParserConfigurationException e) {
			logger.log(Level.SEVERE, e.getLocalizedMessage());
		} catch (SAXException e) {
			logger.log(Level.SEVERE, e.getLocalizedMessage());
		} catch (IOException e) {
			logger.log(Level.SEVERE, e.getLocalizedMessage());
		} catch (TransformerException e) {
			logger.log(Level.SEVERE, e.getLocalizedMessage());
		}
		// We reset the preambule ( but we do not use clear to allow reuse of
		// the previous preambule by the caller )
		if ( this.getPreambule() != null && this.getPreambule().getParams() != null )
			this.setPreambule(new Preambule());
		return result;
	}

	/**
	 * @return the xmlResult
	 */
	public Result getXmlResult() {
		return xmlResult;
	}

	/**
	 * @param xmlResult the xmlResult to set
	 */
	public void setXmlResult(Result xmlResult) {
		this.xmlResult = xmlResult;
	}

	/**
	 * @return the preambule
	 */
	public Preambule getPreambule() {
		return preambule;
	}

	/**
	 * @param preambule the preambule to set
	 */
	public void setPreambule(Preambule preambule) {
		this.preambule = preambule;
	}

	/**
	 * @param return entityResolver
	 */
	public static EntityResolver getEntityResolver() {
	    return entityResolver;
	}

	/**
	 * <p>Set the entityResolver.</p> 
	 * @param entityResolver
	 */
	public static void setEntityResolver(EntityResolver entityResolver) {
	    XSLTMerger.entityResolver = entityResolver;
	}


	/**
	 * @param return uriResolver
	 */
	public static URIResolver getUriResolver() {
	    return uriResolver;
	}

	/**
	 * <p>Set the uriResolver.</p> 
	 * @param uriResolver
	 */
	public static void setUriResolver(URIResolver uriResolver) {
	    XSLTMerger.uriResolver = uriResolver;
	}


}
