/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.ant;

import java.util.ArrayList;
import java.util.List;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.dynamics.Dynamics;
import org.sourceforge.xradar.dynamics.Input;

/**
 * This class is a simple wrapper for Statics merge for Ant.
 * Xml configuration will be :
 *
 *<code>
 	<xradar-dynamics	debug="false"
							config="etc/radar-config.xml"
							target="${docs.home}"
							staticsConfigured="true"> <!--  false will render staticsRoot attributes optionnal -->
			<!-- If orderId == 1, 'previousReport' is not required as it is the first version of the project 
				Xradar will use the etc/config/dynamics/dynamics-master-base.xml
				embedded in the jar distribution.
			-->
			<input		    reportToMerge="release1/docs/xml/radar_report_normalized.xml"
							qualityReportPath="release1/docs/xml/reports/"
							versionId="1"
							daysSinceLast="1"
							staticsRoot="release1/docs"
							masterFinal="${docs.home}/xml/dynamics-master-final.xml"/>
			
			--release1.2 => version = 1 
			
			<input		    previousReport="${docs.home}/xml/dynamics-master-final.xml"
							reportToMerge="release2/docs/xml/radar_report_normalized.xml"
							qualityReportPath="release2/docs/xml/reports/"
							versionId="2"
							daysSinceLast="1"
							staticsRoot="release2/docs"
							masterFinal="${docs.home}/xml/dynamics-master-final.xml"/>
							

		</xradar-dynamics>
	</code>
 *
 * @author Romain PELISSE, belaran@gmail.com
 *
 */
public class DynamicsTask extends AbstractXRadarAntTask
{

	/*
	 * Fields accessible through getter/setters
	 */
	private boolean debug = false;
	private String config;
	private String target;		
	private boolean staticsConfigured = true;
	
	/*
	 * Included element
	 */
	private List<Input> inputs = new ArrayList<Input>(0);
	
	/*
	 * Internal fields
	 */
	private Dynamics	dynamicsEngine;

	
	public void addInput(Input input)
	{
		if ( this.inputs == null )
			this.inputs = new ArrayList<Input>();
		this.inputs.add(input);
	}
	/**
	 * Execute the merges of the reports
	 */
	public void execute() throws BuildException
	{
		validate();
		dynamicsInitializtion();
		try 
		{
			for ( Input input : this.inputs )
			{
				
				// Version specific settings
				dynamicsEngine.setSequenceId(String.valueOf(input.getVersionId()));
				dynamicsEngine.setDaysSinceLast(String.valueOf(input.getDaysSinceLast()));
				dynamicsEngine.setQualityReportPath(input.getQualityReportPath());
				dynamicsEngine.setMasterFinal(input.getMasterFinal());
				if ( isStaticsConfigured() )
					dynamicsEngine.setStaticRoot(input.getStaticsRoot());

				// Updating file from previous version to the current version
				log("Calling execute merge with " + input.getPreviousReport() + " and " + input.getReportToMerge(),Project.MSG_VERBOSE);			
				this.dynamicsEngine.executeMerge(input.getPreviousReport(),input.getReportToMerge());
				// Post processing the previousVersion
				log("Calling post process with " +  input.getMasterFinal(),Project.MSG_VERBOSE);			
				this.dynamicsEngine.postProcess(input.getMasterFinal());
				// Adding static images
				log("Adding static images to website",Project.MSG_VERBOSE);			
				this.dynamicsEngine.copyImagesToWebSite();
				// Generating the dynamics 'website'
				log("Calling dynamics style report with " +  input.getMasterFinal(),Project.MSG_VERBOSE);			
				this.dynamicsEngine.dynamicsStyleReport(input.getMasterFinal(),this.target + Dynamics.EMPTY);
			}
			
		}
		catch (XRadarException e) 
		{
			throw new BuildException(e);
		}

	}

	private void dynamicsInitializtion() throws BuildException
	{
		try 
		{
			this.dynamicsEngine.init();
		} 
		catch (XRadarException e) 
		{
			throw new BuildException(e);
		}
	}

	private void validate() throws BuildException
	{
		// attribute validation
		if ( "".equals(this.getConfig()) )
			throw new BuildException("Config field is not optionnal.");
		if ( "".equals(this.getTarget()) )
			throw new BuildException("Target field is not optionnal.");
		// Validating 'input' reports
		if ( this.inputs.size() <= 0 )
			throw new BuildException("At least one element <input> is required !");
		// Logging settings
		log("Debug:" + this.debug, Project.MSG_VERBOSE);
		log("Target:" + this.target, Project.MSG_VERBOSE);
		log("Config:" + this.config, Project.MSG_VERBOSE);
		log("Is Statics configured ?" + this.isStaticsConfigured(),Project.MSG_VERBOSE);
		// Validating each report
		for ( Input input : this.inputs )
			validate(input);
		// Setting the engine
		this.dynamicsEngine = new Dynamics();
		this.dynamicsEngine.initializedXmlCatalog((! super.isOnline()));
		// Setting (and creating docs-home)
		this.dynamicsEngine.setDebug(this.debug);
		this.dynamicsEngine.setConfig(this.config);
		try 
		{
			this.dynamicsEngine.setDocsHome(this.target);
		}
		catch (XRadarException e) 
		{
			throw new BuildException(e);
		}
	}

	private void validate(Input input) throws BuildException 
	{
		String mssg = "Configure input with versionId" + input.getVersionId();
		// Previous version filename not setted
		if ( input.getPreviousReport() == null || "".equals(input.getPreviousReport()) )
		{
			// VersionId is 1, using default file
			if ( input.getVersionId() == 1 )
					input.setPreviousReport(Dynamics.DEFAULT_BASE_MASTER_REPORT);
			else // this is an error...
				throw new BuildException("PreviousReport field is not optionnal." + mssg);			
		}
		if ( input.getReportToMerge() == null || "".equals(input.getReportToMerge()))
			throw new BuildException("Report-to-merge field is not optionnal." + mssg );
		if ( input.getQualityReportPath() == null && "".equals(input.getQualityReportPath() ) )
			throw new BuildException("qualityReportPath field is not optionnal."+ mssg );
		if ( "".equals(input.getDaysSinceLast()) )
			input.setDaysSinceLast(0);
		if ( "".equals(input.getVersionId()) )
			input.setVersionId(1);
		if ( input.getMasterFinal() == null || "".equals(input.getMasterFinal()) )
			throw new BuildException("masterFinal field is not optionnal." + mssg );
		if ( this.isStaticsConfigured() )
		{
			if ( "".equals(input.getStaticsRoot()) )
				throw new BuildException("Statics root is required if staticConfigured is set to true." + mssg );
			//TODO: Check that directory exists and is readable ! 
			//TODO: Also check for an index.html file to assert if the report was successfully generated ?

		}
		log("Following input is valid:" + input.toString(),Project.MSG_VERBOSE);
	}

	/**
	 * @return the debug
	 */
	public boolean isDebug() {
		return debug;
	}

	/**
	 * @param debug the debug to set
	 */
	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	/**
	 * @return the target
	 */
	public String getTarget() {
		return target;
	}

	/**
	 * @param target the target to set
	 */
	public void setTarget(String target) {
		this.target = target;
	}

	/**
	 * @return the config
	 */
	public String getConfig() {
		return config;
	}

	/**
	 * @param config the config to set
	 */
	public void setConfig(String config) {
		this.config = config;
	}
	/**
	 * @return the staticsConfigured
	 */
	public boolean isStaticsConfigured() {
		return staticsConfigured;
	}

	/**
	 * @param staticsConfigured the staticsConfigured to set
	 */
	public void setStaticsConfigured(boolean staticsConfigured) {
		this.staticsConfigured = staticsConfigured;
	}

}
