/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.ant;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.config.ReleaseData;
import org.sourceforge.xradar.config.ReleaseDataLoader;
import org.sourceforge.xradar.statics.MergeProduct;
import org.sourceforge.xradar.statics.Param;
import org.sourceforge.xradar.statics.Report;
import org.sourceforge.xradar.statics.Statics;
import org.sourceforge.xradar.statics.renderers.XslHtmlWebsite;

/**
 * This classe is a simple wrapper for Statics merge for Ant.
 * Xml configuration will be :
 *
	<xradar-statics debug="false"
      					properties="${basedir}/etc/release1.properties"
      					config="etc/radar-config.xml"
      					target="${basedir}/release1/docs/"
      					javadocEnabled="true"
      					javadocRoot="${basedir}/release1/docs/api"
      					java2htmlRoot="${basedir}/release1/docs/xml/htmlcode">
  			<report file="${jdepend.report.file}"					type="jdepend"/>
  			<report file="${cpd.report.file}"						type="cpd"/>
  			<report file="${javancss.report.file}"					type="javancss"/>
			<report file="${ckjm.report.file}"						type="ckjm"/>
  			<report file="${depfind.report.file}"					type="depfind"/>
  			<report file="${docs.home}/xml/test_depfind_report.xml"	type="test_depfind"/>
			<report file="${checkstyle.report.file}"				type="checkstyle"/>
			<!--
				TODO:
		  			<report file=""	type="bcc"/>
					<report file=""	type="junit"/>
					<report file=""	type="cobertura"/>
			-->
  		</xradar-statics>
 *
 * Order id as xslt fields, if not set ( most likely) will be defines using the type of the tools.
 *
 * 
 * @author Romain PELISSE, belaran@gmail.com
 *
 */
public class StaticsTask extends AbstractXRadarAntTask
{

	/*
	 * Fields accessible through getter/setters
	 */
	private List<Report> reports;
	private boolean debug;
	private String properties;
	private String target;
	private String config;
	private String javadocEnabled = "false";
	private String javadocRoot;
	private String java2htmlRoot;

	/*
	 * Internal fields
	 */
	private XslHtmlWebsite staticEngine;

	public void addReport(Report report)
	{
		if ( this.reports == null )
			this.reports = new ArrayList<Report>();
		this.reports.add(report);
	}

	/**
	 * Execute the merges of the reports
	 */
	@Override
	public void execute() throws BuildException
	{
		validate();
		try
		{
			// Merging the report
			String mergedReport = this.staticEngine.executeMerge();
			log("XML merging result is :\n" + mergedReport,Project.MSG_VERBOSE);
			log("Merging data from tool's reports.", Project.MSG_INFO);
			// Preparing the post processing
			Param radarConfig = new Param(Statics.XRADAR_CONFIG_FILE,config);
			ReleaseData releaseData = new ReleaseDataLoader(properties).load();
			// Post processing
			log("Post processing merged files.", Project.MSG_INFO);
			MergeProduct products = this.staticEngine.executePostProcessing(mergedReport, releaseData, radarConfig);
			// Generating the website
			Map<String,Param> params = new HashMap<String,Param>();
			params.put(Statics.JAVADOC_ENABLED,new Param(Statics.JAVADOC_ENABLED,this.getJavadocEnabled()));
			params.put(Statics.JAVADOC_ROOT,new Param(Statics.JAVADOC_ROOT,this.getJavadocRoot()));
			params.put(Statics.JAVA2HTML_ROOT,new Param(Statics.JAVA2HTML_ROOT,this.getJava2htmlRoot()));
			params.put(Statics.XRADAR_CONFIG_FILE,radarConfig);
			params.put(Statics.LOG,new Param(Statics.LOG,String.valueOf(this.debug)));
			log("Generating static analysis website.", Project.MSG_INFO);
			// Still buggy, only works for images generation !
			this.staticEngine.generateHtmlReport(products, params);
			// Copying required resources
			this.staticEngine.copyStaticResourcesToWebSite(this.getTarget(),this.getTarget() + "/xml/api"); // FIXME: externalize this xml/api?
		}
		catch (XRadarException e)
		{
			throw new BuildException(e);
		}
	}


	/**
	 * Checks if settings are proper, such as:
	 * <ul>
	 * 		<li>set docs home directory based on target attribute ( TODO: check if exist)</li>
	 * 		<li>set debug level based on debug attribute</li>
	 * 		<li>TODO:Report file exists and can be read</li>
	 * 		<li>If not defined load default value for order-id and xslt file</li>
	 * 		<li>TODO:Xslt file and can be accessed through</li>
	 * 		<li>... more ?</li>
	 * </ul>
	 * @throws BuildException
	 *
	 */
	private void validate() throws BuildException
	{
		// attribute validation
		if ( "".equals(this.getTarget()) )
			throw new BuildException("Target field is not optionnal.");
		if ( "".equals(this.getProperties()) )
			throw new BuildException("Properties field is not optionnal.");
		if ( "".equals(this.getConfig()) )
			throw new BuildException("Config field is not optionnal.");
		if ( "".equals(this.getJava2htmlRoot()) )
			throw new BuildException("Java2htmlRoot field is not optionnal.");
		if ( "".equals(this.getJavadocRoot()) && Boolean.valueOf(this.javadocEnabled) )
			throw new BuildException("JavadocRoot field is not optionnal, if javadocEnabled is set to true.");

		// Logging settings
		log("Debug:"+ this.debug, Project.MSG_VERBOSE);
		log("Target:"+ this.target, Project.MSG_VERBOSE);
		log("Properties:"+ this.properties, Project.MSG_VERBOSE);
		log("Config:"+ this.config, Project.MSG_VERBOSE);
		// Setting the engine
		this.staticEngine = new XslHtmlWebsite();
		this.staticEngine.initializedXmlCatalog(super.isOffline());
		this.staticEngine.setDebug(this.debug);
		this.staticEngine.setConfig(this.config);
		// Setting (and creating docs-home)
		try
		{
			this.staticEngine.setDocsHome(this.getTarget());
		}
		catch (XRadarException e)
		{
			throw new BuildException(e);
		}
		// Adding and validating the reports ( validation delegated to the engine)
		for ( Report report : reports )
		{
			// Logging the added report data
			log("Report:"+ report.getType(), Project.MSG_VERBOSE);
			log("->Input file:" + report.getFile(), Project.MSG_VERBOSE);
			// Adding the report
			try
			{
				if ( ! this.staticEngine.addReport(report) )
					log("Report " + report.getType() + " was not added, because " + report.getFile() + " does not exist or is not readable.",Project.MSG_WARN);
			}
			catch (XRadarException e)
			{
				throw new BuildException(e);
			}
		}
	}



	/**
	 * @return the reports
	 */
	public List<Report> getReports() {
		return reports;
	}

	/**
	 * @param reports the reports to set
	 */
	public void setReports(List<Report> reports) {
		this.reports = reports;
	}

	/**
	 * @return the debug
	 */
	public boolean isDebug() {
		return debug;
	}

	/**
	 * @param debug the debug to set
	 */
	public void setDebug(boolean debug) {
		this.debug = debug;
	}

	/**
	 * @return the properties
	 */
	public String getProperties() {
		return properties;
	}

	/**
	 * @param properties the properties to set
	 */
	public void setProperties(String properties) {
		this.properties = properties;
	}

	/**
	 * @return the target
	 */
	public String getTarget() {
		return target;
	}

	/**
	 * @param target the target to set
	 */
	public void setTarget(String target) {
		this.target = target;
	}

	/**
	 * @return the config
	 */
	public String getConfig() {
		return config;
	}

	/**
	 * @param config the config to set
	 */
	public void setConfig(String config) {
		this.config = config;
	}

	/**
	 * @return the java2htmlRoot
	 */
	public String getJava2htmlRoot() {
		return java2htmlRoot;
	}

	/**
	 * @param java2htmlRoot the java2htmlRoot to set
	 */
	public void setJava2htmlRoot(String java2htmlRoot) {
		this.java2htmlRoot = java2htmlRoot;
	}

	/**
	 * @return the javadocEnabled
	 */
	public String getJavadocEnabled() {
		return javadocEnabled;
	}

	/**
	 * @param javadocEnabled the javadocEnabled to set
	 */
	public void setJavadocEnabled(String javadocEnabled) {
		this.javadocEnabled = javadocEnabled;
	}

	/**
	 * @return the javadocRoot
	 */
	public String getJavadocRoot() {
		return javadocRoot;
	}

	/**
	 * @param javadocRoot the javadocRoot to set
	 */
	public void setJavadocRoot(String javadocRoot) {
		this.javadocRoot = javadocRoot;
	}
}
