/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.dynamics;

import java.io.InputStream;
import java.io.Writer;
import java.util.logging.Logger;

import org.sourceforge.xradar.AbstractProcess;
import org.sourceforge.xradar.DefaultReportValues;
import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.statics.Param;
import org.sourceforge.xradar.statics.Preambule;
import org.sourceforge.xradar.util.FileUtils;
import org.sourceforge.xradar.util.StreamUtils;
import org.sourceforge.xradar.util.XRadarUtils;
import org.xml.sax.InputSource;

/**
 * <p>Implements the Dynamic part of the XRadar QA report.</p>
 * 
 * @author Romain Pelisse, belaran@gmail.com
 *
 */
public class Dynamics extends AbstractProcess 
{
	private static final Logger logger = Logger.getLogger(Dynamics.class.getSimpleName());
	
	private static final String REPORT_TO_MERGE = "report-to-merge";
	private static final String QUALITY_REPORT_PATH = "quality-report-path";
	private static final String SEQUENCE_ID = "sequence-id";
	private static final String DAYS_SINCE_LAST ="days-since-last";

	/*
	 * Stylesheets to use
	 */
	private static final String DYNAMICS_STYLE_REPORT = DefaultReportValues.getString("radar.internal.dynamics.process.style.report");;
	private static final String NORMALIZED_PROCESS = DefaultReportValues.getString("radar.internal.dynamics.process.merge.normalized");
	private static final String POST_PROCESSING = DefaultReportValues.getString("radar.internal.dynamics.post.processing");

	public static final String SEQUENCE_ID_ONE = "1";
	public static final String DAYS_SINCE_LAST_DEFAULT = "0";
	public static final String DEFAULT_BASE_MASTER_REPORT = "xradar://resources/etc/config/dynamics/dynamics-master-base.xml";
	public static final String EMPTY = "empty.html";
	private static final String STATICS_CONFIGURED = "statics-configured";
	private static final String STATICS_ROOT = "statics-root";


	private String qualityReportPath;
	private String sequenceId;
	private String daysSinceLast;
	private String masterFinal;
	private String staticRoot = null;

	private Preambule preambule;
	private String mergedFile;

	/**
	 * <p>Default constructor:</p>
	 * <ol>
	 * 	<li>Get logger</li>
	 *  <li>Initialize catalog use to false (default setting)</li>
	 * <ol>
	 * 
	 */
	public Dynamics()
	{
		super.initializedXmlCatalog(false);
	}


	public void init() throws XRadarException
	{
		if ( ! "".equals(this.getDocsHome() ) )
		{
			// Creating the xml directory, along with the docs-home directory
			FileUtils.createDir(this.getDocsHome() + SYSTEM_FILE_SEPARATOR + "xml" + SYSTEM_FILE_SEPARATOR);
			// Creating the img directory 
			FileUtils.createDir(this.getDocsHome() + SYSTEM_FILE_SEPARATOR + "img" + SYSTEM_FILE_SEPARATOR);			
		}
		else
			throw new XRadarException("DocsHome parameter must be setted before called to init().");
	}

	/**
	 * 
	 * 
	 * @param previousVersion
	 * @param currentVersion
	 * @return complete filename to product of the merge
	 * @throws XRadarException
	 */
	public void executeMerge(String previousVersion, String currentVersion) throws XRadarException
	{
		String product = this.getDocsHome() + SYSTEM_FILE_SEPARATOR + "tmp.xml";
		logger.info("Merging previous version file," + previousVersion + ", with " + currentVersion + " file, using " + product + " as a temporary buffer");
		this.preambule = new Preambule();
		preambule.getParams().put(Dynamics.XRADAR_CONFIG_FILE, new Param(Dynamics.XRADAR_CONFIG_FILE,super.getConfig()));
		preambule.getParams().put(Dynamics.REPORT_TO_MERGE, new Param(Dynamics.REPORT_TO_MERGE,currentVersion));
		preambule.getParams().put(Dynamics.QUALITY_REPORT_PATH, new Param(Dynamics.QUALITY_REPORT_PATH,this.qualityReportPath));
		preambule.getParams().put(Dynamics.DAYS_SINCE_LAST, new Param(Dynamics.DAYS_SINCE_LAST,this.daysSinceLast));
		preambule.getParams().put(Dynamics.SEQUENCE_ID, new Param(Dynamics.SEQUENCE_ID,this.sequenceId));
		merger.setXmlResult(StreamUtils.createOuput(StreamUtils.createFileWriter(product)));
		merger.setPreambule(preambule);
		logger.fine("Executing merge now");
		// FIXME: this is not really elegant
		InputStream input = null;
		if ( previousVersion.contains(XRADAR_PROTOCOL) )
			input = StreamUtils.retrieveFileContentAsStream(previousVersion);
		else
			input = StreamUtils.readFile(previousVersion);
		// Merging..
		merger.merge(new InputSource(input),StreamUtils.retrieveFileContentAsStream(NORMALIZED_PROCESS));
		// Processing file
		logger.fine("Merging finished, saving result in file " + product);
		// If this is 'base' file from xradar we have to change the value of previousVersion to "real" file
		if ( DEFAULT_BASE_MASTER_REPORT.equals(previousVersion) )
			previousVersion = this.getMasterFinal();
		this.mergedFile = previousVersion;
		// Now we update the previous version file with the product of the merge
		StreamUtils.copyFileToFile(product,previousVersion);
		// products may be now deleted
		FileUtils.deleteFile(product);
		// Processing file
		logger.fine("Deleting temporaty file " + product);

	}

	/** 
	 * <p>Post process de dynamics merge report.</p>
	 * 
	 * @param output
	 * @return complete filename to product of the post processing.
	 * @throws XRadarException
	 */
	public void postProcess(String output) throws XRadarException
	{
		logger.info("Post processsing file " + output);
		// Verifying methods arguments
		if ( "".equals(output) && ! "".equals(this.getMasterFinal()) )
			output = this.getMasterFinal();
		// Processing file
		String tmpFile = FileUtils.addSeparatorIfMissing(this.getDocsHome()) + "tmp.xml";
		merger.setXmlResult(StreamUtils.createOuput(StreamUtils.createFileWriter(tmpFile)));
		logger.fine("Post processing input file " + this.mergedFile + ".");
		merger.merge(new InputSource(StreamUtils.readFile(this.mergedFile)),StreamUtils.retrieveFileContentAsStream(POST_PROCESSING));
		logger.fine("Result saved in " + output);
		StreamUtils.copyFileToFile(tmpFile,output);
	}

	public void copyImagesToWebSite() throws XRadarException
	{
		this.copyImagesToWebsite(this.getDocsHome() + SYSTEM_FILE_SEPARATOR);
	}

	public void dynamicsStyleReport(String input, String output) throws XRadarException
	{
		logger.info("Generating website from file " + output);
		logger.fine("dynamicsStyleReport called.");
		// Creating the output writer
		Writer writer = StreamUtils.createFileWriter(output);
		merger.setXmlResult(StreamUtils.createOuput(writer));
		// Defining the transformation preambule
		// Note that we reuse preambule from previous operation
		logger.fine("isStaticRoot enabled:" + this.getStaticRoot());
		if ( this.getStaticRoot() != null && ! "".equals(this.getStaticRoot()) ) 
		{
			this.preambule.getParams().put(STATICS_CONFIGURED, new Param(STATICS_CONFIGURED,"true"));
			this.preambule.getParams().put(STATICS_ROOT, new Param(STATICS_ROOT,this.getStaticRoot()));
		}
		else
			this.preambule.getParams().put(STATICS_CONFIGURED, new Param(STATICS_CONFIGURED,"false"));
		
		if (XRadarUtils.isRunningOnWindowsOS()) {
			preambule.getParams().put(Dynamics.DOCS_HOME, new Param(Dynamics.DOCS_HOME,SYSTEM_FILE_SEPARATOR + super.getDocsHome()));
		} else {
			preambule.getParams().put(Dynamics.DOCS_HOME, new Param(Dynamics.DOCS_HOME,super.getDocsHome()));
		}
		merger.setPreambule(this.preambule);
		// execute the operation
		logger.fine("Executing the transformation.");
		merger.merge(new InputSource(StreamUtils.readFile(input)),StreamUtils.retrieveFileContentAsStream(DYNAMICS_STYLE_REPORT));
		// Cleaning dynamics 'state'
		this.reset();
	}

	/**
	 * This object keep some kind of 'state', after the call of the following methods:
	 * <ol>
	 * 		<li>executeMerge</li>
	 * 		<li>dynamicsStyleReport</li>
	 * </ol>
	 * To clean the instance ( ensure that not state is conserved, you may call this method.
	 * However, note that this method is already called at the end of the dynacsStyleReport method.
	 */
	public void reset()
	{
		// We remove all the parameter
		if ( this.preambule != null && this.preambule.getParams() != null )
			this.preambule.getParams().clear();
		// We also set to null to come back to 'init' state
		this.preambule = null;
		this.staticRoot = null;
		this.masterFinal = null;
		this.qualityReportPath = null;
	}

	/**
	 * @return the masterFinal
	 */
	public String getMasterFinal() {
		return masterFinal;
	}
	/**
	 * @param masterFinal the masterFinal to set
	 */
	public void setMasterFinal(String masterFinal) {
		this.masterFinal = masterFinal;
	}

	/**
	 * @return the sequenceId
	 */
	public String getSequenceId() {
		return sequenceId;
	}

	/**
	 * @param sequenceId the sequenceId to set
	 */
	public void setSequenceId(String sequenceId) {
		this.sequenceId = sequenceId;
	}

	/**
	 * @return the daysSinceLast
	 */
	public String getDaysSinceLast() {
		return daysSinceLast;
	}

	/**
	 * @return the qualityReportPath
	 */
	public String getQualityReportPath() {
		return qualityReportPath;
	}

	/**
	 * @param qualityReportPath the qualityReportPath to set
	 */
	public void setQualityReportPath(String qualityReportPath) {
		this.qualityReportPath = qualityReportPath;
	}

	/**
	 * @param daysSinceLast the daysSinceLast to set
	 */
	public void setDaysSinceLast(String daysSinceLast) {
		this.daysSinceLast = daysSinceLast;
	}


	/**
	 * @return the staticRoot
	 */
	public String getStaticRoot() {
		return staticRoot;
	}
	/**
	 * @param staticRoot the staticRoot to set
	 */
	public void setStaticRoot(String staticRoot) {
		this.staticRoot = staticRoot;
	}	
}
