/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.dynamics;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.logging.Logger;

import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.util.FileUtils;

/**
 * <p>Command line version of XRadar to have dynamic reports</p>
 * 
 * @author Romain PELISSE, belaran@gmail.com
 * @author Zahir Nuckchady, zahir.nu@gmail.com
 * 
 */
public class XRadarDynamics {

	private static final Logger logger = Logger.getLogger(XRadarDynamics.class.getSimpleName());
	/**
	 *Command line to use XRadar
	 *@param args, representing the arguments passed in the command line
	 * @throws XRadarException  
	 */
	public static void runDynamics(String[] args) throws XRadarException {
		List<String> arguments = Arrays.asList(args);
		DynamicsOptionalParameters paramters = new DynamicsOptionalParameters();
		List <String> argsWithoutOptions = removeOptional(arguments,paramters);
		checkReleaseArgs(argsWithoutOptions);
		List<Input> inputs = retrieveEachReleaseData(arguments);
		Dynamics dynamicsEngine = new Dynamics();
		for ( Input input : inputs) {
			String masterFinalReport = FileUtils.addSeparatorIfMissing(paramters.getTarget()) +  input.getMasterFinal();
			// Version specific settings
			dynamicsEngine.setSequenceId(String.valueOf(input.getVersionId()));
			dynamicsEngine.setDaysSinceLast(String.valueOf(input.getDaysSinceLast()));
			dynamicsEngine.setQualityReportPath(input.getQualityReportPath());
			dynamicsEngine.setMasterFinal(masterFinalReport);
			dynamicsEngine.setDocsHome(paramters.getTarget());
			if ( XRadarDynamics.isStaticConfigured(arguments) )
				dynamicsEngine.setStaticRoot(input.getStaticsRoot());
			// Updating file from previous version to the current version
			logger.info("Calling execute merge with " + input.getPreviousReport() + " and " + input.getReportToMerge());
			dynamicsEngine.executeMerge(input.getPreviousReport(), input.getReportToMerge());
			// Post processing the previousVersion
			logger.info("Calling post process with " + masterFinalReport);			
			dynamicsEngine.postProcess(masterFinalReport);
			// Adding static images
			logger.info("Adding static images to website");			
			dynamicsEngine.copyImagesToWebSite();
			// Generating the dynamics 'website'
			logger.info("Calling dynamics style report with " +  input.getMasterFinal());
			dynamicsEngine.dynamicsStyleReport(masterFinalReport, FileUtils.addSeparatorIfMissing(paramters.getTarget()) + Dynamics.EMPTY);
		}
	}


	/**
	 *	<p>Verifies if any optional arguments are present and removes them from the list of arguments passed.</p>
	 *
	 *	@param args, representing the arguments passed in the command line
	 */
	private static List <String> removeOptional (List<String> arguments,DynamicsOptionalParameters dynamicsParams)
	{
		List<String> releaseArguments = new ArrayList<String>(arguments); 
		for(String argument :arguments){
			if ( argument.contains("dynamic") ) {
				releaseArguments.remove(argument);	
			}
			else if ( argument.contains("--debug") ) {
				dynamicsParams.setDebug(true);
				releaseArguments.remove(argument);	
			}
			else if ( argument.contains("--staticsConfigured") ) {
				dynamicsParams.setStaticsConfigured(true);
				releaseArguments.remove(argument);	
			}
			else
			{
				boolean isConfig =  argument.contains("--config");
				boolean isTarget =  argument.contains("--target");
				if ( isConfig || isTarget ) {
					String value = arguments.get(arguments.indexOf(argument) + 1);
					if ( isConfig && fileExists(value,argument) )
						dynamicsParams.setConfig(value);
					if ( isTarget )
						dynamicsParams.setTarget(FileUtils.addSeparatorIfMissing(value));
					releaseArguments.remove(argument);
					releaseArguments.remove(value);	
				}
			}
		}
		// Checks to see if compulsory arguments are present and intialized
		if ( dynamicsParams.getTarget() == null || "".equals(dynamicsParams.getTarget()) ||
				dynamicsParams.getConfig() == null || "".equals(dynamicsParams.getConfig())) {
			throw new IllegalArgumentException("Compulsory arguments are: --target and --config missing");
		}
		return releaseArguments;
	}

	/**
	 * <p>Verifies if all the arguments necessary are presents.</p>
	 * 
	 *@param args, representing the arguments passed in the command line
	 *@throws IllegalArgumentException if compulsory arguments are missing
	 */
	private static void checkReleaseArgs(List<String> arguments) {
		String [] requiredArgument = new String [] {"--qualityReportPath","--daysSinceLast","--masterFinal","--previousReport","--reportToMerge"};
		for(String argument : requiredArgument){
			// Test to see if the 1st argument is the --release command
			if( ! arguments.contains(argument)){
				throw new IllegalArgumentException(argument + " is required.");
			}
		}
		
		if (! (arguments.get(0).startsWith("--release")) ){
			throw new IllegalArgumentException("It seems the --release option is missing, find this " +arguments.get(0) + " instead."); 
		}
	}

	private static boolean isStaticConfigured(List<String> arguments) {
		boolean status = false;
		final String STATIC_ROOT_ARGS = "--staticsRoot";
		if (  arguments.contains(STATIC_ROOT_ARGS) ) {
			// Ok arg is present, but does it have a value ?
			int argValueId = arguments.indexOf(STATIC_ROOT_ARGS) + 1;
			if ( argValueId > arguments.size() )
				throw new IllegalArgumentException(STATIC_ROOT_ARGS + "  requires a value !");
			String value = arguments.get(argValueId);
			if ( value == null || "".equals(value) || ! new File(value).exists() && ! new File(value).isDirectory() )
				throw new IllegalArgumentException(STATIC_ROOT_ARGS + " value must be a valid directory.");
		}
		return status;
	}

	/**
	 *Verifies if the file exists
	 *@param filename and Command
	 */
	private static boolean fileExists (String filename, String command) {
		if ( ! FileUtils.exists(filename) ) {
			throw new IllegalArgumentException("File for " + command + " is missing or does not exist");
		}
		return true;
	}
	
	/**
	 *Verifies if the first element is the release command and initializes the release arguments
	 *@param args, representing the arguments passed in the command line
	 * @throws XRadarException 
	 */
	private static List<Input> retrieveEachReleaseData(List<String> arguments) throws XRadarException 
	{	
		List<Input> inputs = new ArrayList<Input>();
		int firstReleaseId;
		int nextReleaseId;
		int idRelease = 1;
		
		do 
		{
			firstReleaseId = arguments.indexOf("--release" +  idRelease);
			nextReleaseId = arguments.indexOf("--release" + (idRelease + 1) ) ;
			if ( firstReleaseId != -1 )
			{
				if ( nextReleaseId == -1 ) 
					nextReleaseId = arguments.size() - 1;
				List<String> releaseSubList = arguments.subList(firstReleaseId + 1, nextReleaseId);		
				Input dynamicsReleaseParams = new Input();
				dynamicsReleaseParams.setVersionId(idRelease);
				for ( String argument : releaseSubList ) 
				{
					int idNumber = releaseSubList.indexOf(argument); 
					int idValue = idNumber + 1;
					if ( idValue < releaseSubList.size() )
					{
						if (argument.contains ("--daysSinceLast"))
							dynamicsReleaseParams.setDaysSinceLast(Integer.parseInt(releaseSubList.get(idValue)));		
						else {
							String value = releaseSubList.get(idValue);
							if (argument.contains ("--previousReport") ) // File may not exists at this point, so no way to check.
								dynamicsReleaseParams.setPreviousReport(value);		
							else if (argument.contains ("--reportToMerge") && fileExists(value, argument))
								dynamicsReleaseParams.setReportToMerge(value);		
							else if (argument.contains ("--qualityReportPath") && fileExists(value, argument) )
								dynamicsReleaseParams.setQualityReportPath(FileUtils.addSeparatorIfMissing(value));		
							else if (argument.contains ("--staticsRoot") && fileExists(value, argument))
								dynamicsReleaseParams.setStaticsRoot(value);		
							else if (argument.contains ("--masterFinal")  )
								dynamicsReleaseParams.setMasterFinal(value);		
						}
					}
				}

				if ( dynamicsReleaseParams.getPreviousReport() == null || 
						"".equals(dynamicsReleaseParams.getPreviousReport() ) ) {
					if ( dynamicsReleaseParams.getVersionId() == 1 )
						dynamicsReleaseParams.setPreviousReport(Dynamics.DEFAULT_BASE_MASTER_REPORT);
					else 
						throw new XRadarException("Previous report is required for any release but the first one");
				}
				inputs.add(dynamicsReleaseParams);
				idRelease++;
			}
		}
		while ( firstReleaseId != -1 ) ;
		return inputs;
	}
}
