/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.resolver;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;

/**
 * XRadarResourceLoader handles the so called 'xradar' protocol
 * Basicly, it look for the file in the classpath.
 *
 * @author Romain PELISSE <belaran@gmail.com>
 *
 */
public class XRadarResourceLoader extends URLConnection
{
	private static final String[] windows = { "Windows 98", "Windows ME", "Windows 2000", "Windows XP", "Vista, Windows 2003" };
	private static final Logger logger = Logger.getLogger(XRadarResourceLoader.class.getSimpleName());
	private boolean isRunningOnWindows;

	protected XRadarResourceLoader(URL url)
	{
		super(url);
		//FIXME: not really sure this OS checking code is still usefull
		if ( isRunningOnWindowsOS() )
		{
			isRunningOnWindows = true; 
		}
	}

	private boolean isRunningOnWindowsOS()
	{
		boolean status = false;
		int idWindows;
		String os = System.getProperty("os.name");
		if (os != null )
			for ( idWindows = 0; idWindows < windows.length; idWindows++ )
				if ( os.startsWith(windows[idWindows]) )
					status = true;
		return status;
	}

	@Override
	public void connect() throws IOException
	{
		// Not implemented as xradar:// is a "deconnected protocol"
	}

	/**
	 * Returns input stream
	 *
	 * @return An inputstream to file or null if the file is not found
	 */
	@Override
	public InputStream getInputStream() throws IOException
	{
		logger.log(Level.FINEST,"Resolving URL:" + this.getURL());
		String file = this.getURL().getFile();
		logger.log(Level.FINEST, "Loading file " + file + " from classpath.");
		InputStream stream = this.getClass().getResourceAsStream(file);
		if ( stream == null )
		{
			logger.log(Level.FINEST, "File " + file + " not loaded. Trying file system...");
			// If this failed, we look on the file system...
			stream = this.getInputStreamFromFileSystem(file);
		}
		return stream;
	}

	/**
	 * If we can't file the file from the classpath, we use this method to retrieve it
	 * from the filesystem.
	 *
	 * @param file
	 * @return the inputstream or null.
	 * @throws IOException
	 */
	protected InputStream getInputStreamFromFileSystem(String file) throws IOException
	{
		InputStream stream = null;
		// Hack for windows: testing if canonicalPath return a properly "rooted" path
		if ( isRunningOnWindows )
			file = windowsHack(file);
		File fileDescriptor = new File(file);
		if ( ! fileDescriptor.exists() )
			logger.log(Level.FINER, "File " + file + " does not exist.");
		else
		{
			if ( ! fileDescriptor.canRead() )
				logger.log(Level.FINEST, "File " + file + "can't be read.");
			else
				stream = new FileInputStream(fileDescriptor);
		}
		if ( stream != null )
			logger.log(Level.FINEST,"File " + file + "was found on file system.");
		return stream;
	}

	public String windowsHack(String file) throws IOException
	{
		logger.finest("windows adaptation need for filepath:" + file);
		file = new File(file).getCanonicalPath();
		if ( file.contains("\\") && file.contains("/") )
		{
			// We replace only the '\\' by '/'
			file = file.replace('\\', '/');
			// If this is an absolute path, we remove the drive
			Pattern pattern = Pattern.compile("^[A-Za-z]:.*$");
			if ( pattern.matcher(file).matches() )
			{
				file = file.substring(2,file.length());
			}
		}
		logger.finest("Filepath modified to" + file);
		return file;
	}
}
