
/**
 * 
 */
package org.sourceforge.xradar.results;

import java.io.InputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;

import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.util.FileUtils;
import org.sourceforge.xradar.util.StreamUtils;
import org.sourceforge.xradar.util.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * 
 * @author Romain PELISSE, <belaran@gmail.com>
 */
public class AnalysingProcess {

	private static final Logger logger = Logger.getLogger(AnalysingProcess.class.getSimpleName());
	private List<Violation> violations;

	private Map<String,Analyser> alreadyLoaded = new HashMap<String,Analyser>(0);
	private Map<String,InputStream> resourcesLoaded = new HashMap<String, InputStream>(0);

	private boolean isRuleHasNameAndQueryDefined(Rule rule) {

		return ! "".equals(rule.getName()) && ! "".equals(rule.getQuery()) ;
	}

	public List<Violation> analyseResults(List<Rule> rules) throws XRadarResultsAnalyserException, XRadarException {
		logger.finest("invoked");
		if ( violations == null )
			throw new XRadarResultsAnalyserException("initialization error, please set 'violations' through appropriate getter");
		logger.finest("called with violations=" + violations);
		for (Rule rule : rules ) {
			logger.finest("descriptor:" + rule.toString());
			if ( rule.getType() == null || "".equals(rule.getType() ) )
				throw new XRadarResultsAnalyserException("empty or null  analyser:");
			Analyser analyser = this.getAnalyserInstance(rule);
			violations.addAll(addRuleToViolations(analyser.analyse(new QueryData(rule.getQuery(),loadDataFile(rule.getFile()))),rule));
		}
		return violations;
	}

	private InputStream loadDataFile(String filename) throws XRadarException {
		InputStream inputStream = null;
		if (! resourcesLoaded.containsKey(filename) && FileUtils.exists(filename) ) {
			inputStream = StreamUtils.fromFilenameToInputStream(filename);
			resourcesLoaded.put(filename,inputStream);
		} else
			inputStream = resourcesLoaded.get(filename);
		return inputStream;
	}

	private List<Violation> addRuleToViolations(List<Violation> violations, Rule rule) {
		for ( Violation violation : violations ) {
			violation.setRule(rule);
		}
		return violations;
	}

	public static String findClassNameForType(String type) {
		if ( "xpath".equals(type) )
			return XPathQueryAnalyser.class.getCanonicalName();
		return type;
	}

	public List<Violation> getViolations() {
		return violations;
	}

	public void setViolations(List<Violation> violations) {
		this.violations = violations;
	}

	/**
	 * <p>As analyser implementation are supposed to be rather 'stateless',
	 * they are only instatiate once, and this method provide access to the 
	 * already instantiated Analyser instance (or create the appropriate
	 * instance.</p>
	 *  
	 * @param type
	 * @return the instance associated with the type
	 * @throws XRadarResultsAnalyserException
	 */
	public  Analyser getAnalyserInstance(Rule rule) throws XRadarResultsAnalyserException {
		Analyser analyser = null;

		if ( ! this.alreadyLoaded.containsKey(rule.getType()) ) {
			// Maybe this a predefined rule
			if (!isRuleHasNameAndQueryDefined(rule)){
				RuleExport re = new RuleExport();
				alreadyLoaded.putAll(re.listRule(rule));
				for (int i = 0; i < alreadyLoaded.size(); i++) {
					if (this.alreadyLoaded.containsKey(rule.getType())){
						analyser = this.alreadyLoaded.get(rule.getType());
					}
				}
			}
			// 
			String type = AnalysingProcess.findClassNameForType(rule.getType());
			try {
				analyser = (Analyser) this.getClass().getClassLoader().loadClass(type).newInstance();
			} catch (InstantiationException e) {
				throw new XRadarResultsAnalyserException(e);
			} catch (IllegalAccessException e) {
				throw new XRadarResultsAnalyserException(e);
			} catch (ClassNotFoundException e) {
				throw new XRadarResultsAnalyserException(e);
			}
		}
		else
			analyser = this.alreadyLoaded.get(rule.getType());
		return analyser;

	}
}
