/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.results;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * <p>Service to execute XPath query on the provided input stream.</p>
 * 
 * @author Romain PELISSE, <belaran@gmail.com>
 */
public class XPathQueryAnalyser implements Analyser {
	
	private final static Logger logger = Logger.getLogger(XPathQueryAnalyser.class.getSimpleName());

	private DocumentBuilder builder;
	private XPath xpath;
	private InputStream inputStream;
	private Document doc;
	
	
	/**
	 * @return the inputstream
	 */
	public InputStream getInputStream() {
		return inputStream;
	}

	/**
	 * 
	 * @param inputStream
	 */
	public void setInputStream(InputStream inputStream) {
		this.inputStream = inputStream;
		this.doc = null; // any new inputstream invalids the existing doc, if any.
	}

	public DocumentBuilder getBuilder() {
		return builder;
	}


	public void setBuilder(DocumentBuilder builder) {
		this.builder = builder;
	}


	/**
	 * Provided for commodity, not use if app's client wants to use an other implementation
	 * of document builder.
	 * 
	 * @return a DocumentBuilder instance or null if failed to create it.
	 */
	public DocumentBuilder createDefaultBuilder() {
		DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
		domFactory.setNamespaceAware(true); 
		try {
			return domFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			// This should never happen ?
			logger.severe(e.getMessage());
		}
		return null;			
	}

	/**
	 * @return the xpath
	 */
	public XPath getXpath() {
		return xpath;
	}

	/**
	 * @param xpath the xpath to set
	 */
	public void setXpath(XPath xpath) {
		this.xpath = xpath;
	}

	/*
	 * Return the Document instance, associated to the current InputStream 
	 * @return
	 * @throws XRadarResultsAnalyserException
	 */
	private Document getCurrentDocument() throws XRadarResultsAnalyserException {
		if ( doc == null ) {
			if ( inputStream == null )
				throw new XRadarResultsAnalyserException("No input stream toward XML data provided !");
			else
				try {
					doc = builder.parse(this.inputStream);
				} catch (SAXException e) {
					new XRadarResultsAnalyserException(e);
				} catch (IOException e) {
					new XRadarResultsAnalyserException(e);
				}
		}
		return doc;
	}
	
	/**
	 * <p>Execute the provided query on the configured InputStream. Check 
	 * 
	 * @param src
	 * @param query
	 * @return
	 */
	public List<Violation> analyse(QueryData data) throws XRadarResultsAnalyserException {	
		List<Violation> results = new ArrayList<Violation>(0);
		String query = initialize(data);		
		
		if ( query != null && ! "".equals(query) ) {
			try {
				doc = getCurrentDocument();
				List<Node> nodes = executeXPathQuery(doc, query);
				for (Node node : nodes) {
					results.add(this.nodeToViolation(node));
				}
			} catch (XPathExpressionException e) {
				new XRadarResultsAnalyserException(e);
			}
		}
		return results;
	}
	
	private String initialize(QueryData data) {
		if ( this.builder == null )
			this.builder = createDefaultBuilder();
		if ( this.xpath == null )
			this.xpath = XPathFactory.newInstance().newXPath();
		this.setInputStream(data.getResultsData());
		return data.getQuery();
	}
	
	private Violation nodeToViolation(Node node) {
		Violation violation = new Violation();
		violation.setName(node.getNodeName());
		return violation;
	}

	/*
	 * 
	 */
	public List<Node> executeXPathQuery(Document doc,String query) throws XPathExpressionException {
		List<Node> results = new ArrayList<Node>(0);
		XPathExpression expr = xpath.compile(query);
		Object result = expr.evaluate(doc, XPathConstants.NODESET);
		NodeList nodes = (NodeList) result;
		logger.finest("Query:" + query + " returned " + nodes.getLength() + " items.");
		results = new ArrayList<Node>(nodes.getLength());
		for (int indexItem = 0; indexItem < nodes.getLength(); indexItem++) {
			results.add(nodes.item(indexItem));
		}
		return results;
	}
}
