/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.statics;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.sourceforge.xradar.AbstractProcess;
import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.logging.LogUtils;
import org.sourceforge.xradar.util.FileUtils;
import org.sourceforge.xradar.util.StreamUtils;
import org.sourceforge.xradar.util.URLUtils;
import org.xml.sax.InputSource;

import ccl.util.FileUtil;

/**
 * 
 * @author Romain PELISSE <belaran@gmail.com>
 *
 */
public class MergeReportEngine extends AbstractProcess {

	private static final Logger logger = Logger.getLogger(MergeReportEngine.class.getSimpleName());
	
	public static final String LOG = "log";
	protected static final String INPUT_FILE_FIELD = "input-report-file";

	private boolean unitTestAvailaible = false;
	private SortedSet<Report> reportsToMerge = new TreeSet<Report>();
	private List<String> supportedTools;
	
	protected int nbToolsSupported;
	
	/**
	 * @return the unitTestAvailaible
	 */
	public boolean isUnitTestAvailaible() {
		return unitTestAvailaible;
	}

	/**
	 * @param unitTestAvailaible the unitTestAvailaible to set
	 */
	public void setUnitTestAvailaible(boolean unitTestAvailaible) {
		this.unitTestAvailaible = unitTestAvailaible;
	}

	/**
	 * @return the reportsToMerge
	 */
	public SortedSet<Report> getReportsToMerge() {
		return reportsToMerge;
	}

	/**
	 * @param reportsToMerge the reportsToMerge to set
	 */
	public void setReportsToMerge(SortedSet<Report> reportsToMerge) {
		this.reportsToMerge = reportsToMerge;
	}

	/**
	 * @return the supportedTools
	 */
	public List<String> getSupportedTools() {
		return supportedTools;
	}

	/**
	 * @param supportedTools the supportedTools to set
	 */
	public void setSupportedTools(List<String> supportedTools) {
		this.supportedTools = supportedTools;
	}

	/**
	 * @return the nbToolsSupported
	 */
	public int getNbToolsSupported() {
		return nbToolsSupported;
	}

	/**
	 * @param nbToolsSupported the nbToolsSupported to set
	 */
	public void setNbToolsSupported(int nbToolsSupported) {
		this.nbToolsSupported = nbToolsSupported;
	}

	/**
	 * @return the classesDirectory
	 */
	public String getClassesDirectory() {
		return classesDirectory;
	}

	/**
	 * @param classesDirectory the classesDirectory to set
	 */
	public void setClassesDirectory(String classesDirectory) {
		this.classesDirectory = classesDirectory;
	}

	private String classesDirectory; // Only required if there is no 'jdepend' report to build the backbone file on top of it

	/**
	 * <p>Load from properties file the metadata on each tools XRadar can agregate.</p>
	 */
	protected void loadSupportedToolsList() {
		this.nbToolsSupported = getDefaultIntValue("Statics.tools.nbTools");
		this.supportedTools = new ArrayList<String>(this.nbToolsSupported);
		for ( int toolId = 1; toolId <= this.nbToolsSupported ; toolId++ )
			this.supportedTools.add(getDefault("Statics.tools." + toolId));
	}

	/*
	 * Check if filename is not null or equals to "", then try to use it
	 * as an XRadar URL. 
	 *
	 * @param filename
	 * @param report
	 * @throws BuildException
	 */
	//TODO: refactor this to have the "File handling" part of the code
	// relying more on FileUtils.
	private void checkFile(String filename,Report report) throws XRadarException {
		// weirdly enough, a 'null' value for filename is not an issue...
		// FIXME: Add parameters to message
		if ( "".equals(filename)) 
			throw new XRadarException(getDefault("Statics.message.missingFilename") + report.toString());
		else if ( filename != null )
		{	
			if ( ! filename.contains(XRADAR_PROTOCOL) ) {
				try {
					filename = new File(filename).toURL().toString();
					report.setFile(filename);
				} catch (MalformedURLException e) {
					throw new XRadarException(e);
				}
			}
			if ( URLUtils.openConnection(filename) == null )
				throw new XRadarException("Statics.message.malformedFilename" + filename); // FIXME: Add parameters to message
		}
	}
	
	
	/**
	 * <p>Add a new report prior to merge.</p>
	 * 
	 * @param report
	 * @return
	 * @throws XRadarException
	 */
	public boolean addReport(Report report) throws XRadarException {
		boolean status = false;
		// TYPE Checking if the field type has been properly set
		if ( "".equals(report.getType()) )
			throw new XRadarException(getDefault("Statics.message.missingReportType"));
		else if ( ! this.supportedTools.contains(report.getType()))
			throw new XRadarException(getDefault("Statics.message.unsupportedTool")); // FIXME: Add arguments to message

		// FILENAME: Checking if the file exists and is readable
		checkFile(report.getFile(), report);
		if ( "".equals(report.getFile()))
			throw new XRadarException(getDefault("Statics.message.missingFilename"));
		File input = new File(getFile(report.getFile()));
		if ( input.exists() && input.canRead() )
		{
			// MERGING: Checking if mergingFile is setted
			checkFile(report.getMergingFile(), report);
			if ( ! "".equals(report.getMergingFile() ) )
				report.setMergingFile(getDefault("Statics.defaultXslt." + report.getType()));

			// ORDERID: Order consistency checking
			Integer orderId = Integer.valueOf(report.getOrderId());
			if ( report.getOrderId() == 0 )
				report.setOrderId(getDefaultIntValue("Statics.defaultOrderId." + report.getType()));
			if ( ! this.reportsToMerge.add(report) )
				throw new XRadarException(getDefault("Statics.message.inconsistentOrderId")
						+ orderId + getDefault("Statics.message.OrderIdAlreadyUsed"));
			// Setting the flag if we add the junit report
			if ( getDefault("Statics.tools.9").equals(report.getType()) )
				this.unitTestAvailaible = true;
			status = true;
		}
		return status;
	}
	/*
	 * tmp hack
	 */
	private String getFile(String fileAsAnURL) throws XRadarException {
		try {
			return new URL(fileAsAnURL).getFile();
		} catch (MalformedURLException e) {
			throw new XRadarException(e);
		}
	}

	/**
	 * Executes the merge of each report and then return the product as a String.
	 *
	 * @return the final result, returned as a String.
	 * @throws XRadarException 
	 */
	public String executeMerge() throws XRadarException {

		// Setting the debug level
		if ( this.isDebug() )
			logger.setLevel(Level.FINEST);
		// First merge all the report
		StringWriter currentResult = null;
		// Do we have a JDepent report ?
		createBackboneReportIfNeeded();
		// Now, let merge all the tools reports into one
		for ( Report report : this.reportsToMerge ) {
			if ( this.isDebug() ) {
				logger.fine("Processing report:" + report.getFile() + "[type:" + report.getType() + "]");
				logger.fine("\tXSLT:" 	+ report.getMergingFile());
				logger.fine("\tOrderId:"  + report.getOrderId());
			}
			// TODO: Still a StringWriter here, maybe still source of memory consumption issue ?
			StringWriter writer = new StringWriter();
			merger.setXmlResult(StreamUtils.createOuput(writer)); //new StreamResult(System.out)
			InputSource inputSource;
			// Only on the first merge the report file is the input
			if ( report.getOrderId() == 1 )
				inputSource = new InputSource (URLUtils.openStream(report.getFile()));
			else {
				merger.setPreambule(getPreambule(report));
				String str = currentResult.getBuffer().toString();
				inputSource = new InputSource(new StringReader(str));
			}
			// TODO: Add success/failure checking, semantic-based (Look for specific xml part that 
			// should have been added by the merge ... Maybe using the Analyser API ?
			merger.merge(	inputSource,
					new BufferedInputStream(URLUtils.openStream(report.getMergingFile())));
			currentResult = writer;
		}
		LogUtils.logBuffer(logger,currentResult,"Report produce from merge");
		return currentResult.getBuffer().toString();
	}
	
	private void createBackboneReportIfNeeded() throws XRadarException {
		int backboneReportOrderId = getDefaultIntValue("Statics.defaultOrderId.jdepend");
        if (    this.getClassesDirectory() != null && 
        		! "".equals(this.getClassesDirectory()) && // if we have no classes directory, no way around JDepend !
                backboneReportOrderId >  0 &&
                this.reportsToMerge.contains( new Report( backboneReportOrderId) )  ) {
			String reportFilename = this.getDocsHome() + "backbonefile.xml";
			File backboneReport = new File(this.getDocsHome() + "backbonefile.xml");
			if ( ! MergeFileBackboneCreator.createBackBoneFile( new File(this.getClassesDirectory()), backboneReport) ) //NOPMD (maybe even a PMD Bug)
				throw new XRadarException("No jdepend report provided to build on and the classes directory provided " + this.getClassesDirectory() + " does not seems to exist");
			else
				this.reportsToMerge.add(new Report("jdepend",reportFilename));
		}
	}
	
	/**
	 * <p>Return preambule used for each report merge.</p>
	 * @param report
	 * @return
	 */
	public Preambule getPreambule(Report report) {
		//TODO: To some kind of singleton for this ?
		Preambule preambule = new Preambule();
		preambule.setParams(report.getParams());
		preambule.getParams().put(INPUT_FILE_FIELD,new Param(INPUT_FILE_FIELD,report.getFile()));
		preambule.getParams().put(LOG,new Param(LOG,String.valueOf(this.isDebug())));
		preambule.getParams().put(XRADAR_CONFIG_FILE, new Param(XRADAR_CONFIG_FILE,this.getConfig()));
		return preambule;
	}
}
