/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.statics;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.logging.Logger;

import javax.xml.transform.stream.StreamResult;

import org.sourceforge.xradar.DefaultReportValues;
import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.config.ReleaseData;
import org.sourceforge.xradar.util.FileUtils;
import org.sourceforge.xradar.util.StreamUtils;
import org.sourceforge.xradar.util.XRadarUtils;
import org.xml.sax.InputSource;

/**
 * This classes is "launcher" to execute every merging needed to synthesize the radar-normalized report.
 * Then, it lanches the proper xslt transformation to generate the statics report's website.
 *
 * @author rpelisse, belaran@gmail.com
 *
 */
public class Statics extends MergeReportEngine
{
	private static final Logger logger = Logger.getLogger(Statics.class.getSimpleName());
	// Parameters name for XSLT stylesheets
	public static final	String TEST_ENABLED = "test-enabled";
	public static final String XML_REPORT_DIRECTORY = "target-directory";
	public static final String 	JAVADOC_ENABLED = "java-doc-configured";
	public static final String 	JAVADOC_ROOT = "java-doc-root";
	public static final String 	JAVA2HTML_ROOT = "java2html-root";

	// Internal XSLT stylesheet definition
	private static final String POSTPROCESS = DefaultReportValues.getString("radar.internal.postprocessor");
	private static final String NORMALIZE = DefaultReportValues.getString("radar.internal.normalize.report");
	private static final String QUALITY = DefaultReportValues.getString("radar.internal.finalization");
	private static final String XML_DIRECTORY = "xml";

	private static final String QUALITY_DIRECTORY = "reports";

	private String systemQuality;
	private String normalizedReport;
	private String masterReport;

	/** 
	 * <p>Default constructor:
	 * 		<ol>
	 * 				<li>create logger ;</li>
	 *              <li>initialized XML Catalog ; </li>
	 * 				<li>load supported tools list.</li>
	 *     </ol>
	 * </p>
	 */
	public Statics()
	{
		super.initializedXmlCatalog(true);
		loadSupportedToolsList();
	}

	@Override
	public void setDocsHome(String docsHome) throws XRadarException
	{
		super.setDocsHome(docsHome);
		systemQuality = this.getDocsHome() + SYSTEM_FILE_SEPARATOR + XML_DIRECTORY + SYSTEM_FILE_SEPARATOR + QUALITY_DIRECTORY + SYSTEM_FILE_SEPARATOR + SYSTEM_QUALITY_REPORT;
		normalizedReport = this.getDocsHome() + SYSTEM_FILE_SEPARATOR + XML_DIRECTORY + SYSTEM_FILE_SEPARATOR + NORMALIZED_REPORT_NAME;
		masterReport = this.getDocsHome() + SYSTEM_FILE_SEPARATOR + XML_DIRECTORY + SYSTEM_FILE_SEPARATOR + MASTER_REPORT_NAME;
		if ( this.isDebug() )
		{
			logger.fine("Target files:");
			logger.fine("\tQuality System:" 	+ systemQuality);
			logger.fine("\tNormalized report:"  + normalizedReport);
		}
		createDirectory();
	}

	private void createDirectory() throws XRadarException
	{
		FileUtils.createDir(this.getDocsHome());
		FileUtils.createDir(this.getDocsHome() + SYSTEM_FILE_SEPARATOR + "xml" + SYSTEM_FILE_SEPARATOR);
		FileUtils.createDir(this.getDocsHome() + SYSTEM_FILE_SEPARATOR + "xml" + SYSTEM_FILE_SEPARATOR + "reports" + SYSTEM_FILE_SEPARATOR);
	}

	/*
	 * Create the appropriate XSL Preambule representation based on data extracted
	 * from both ReleaseData object and the RadarConfig one.
	 *
	 * @param releaseData
	 * @param radarConfig
	 * @return
	 */
	private Preambule createPostProcessPreambule(ReleaseData releaseData, Param radarConfig)
	{
		Preambule preambule = new Preambule();
		preambule.getParams().put(radarConfig.getName(),radarConfig);
		preambule.getParams().put(RELEASE_DATE,new Param(RELEASE_DATE,releaseData.getDate()));
		preambule.getParams().put(RELEASE_SYSTEM,new Param(RELEASE_SYSTEM,releaseData.getSystem()));
		preambule.getParams().put(RELEASE_VERSION,new Param(RELEASE_VERSION,releaseData.getVersion()));
		if (XRadarUtils.isRunningOnWindowsOS()) 
			preambule.getParams().put(DOCS_HOME,new Param(DOCS_HOME,SYSTEM_FILE_SEPARATOR + this.getDocsHome()));
		else
			preambule.getParams().put(DOCS_HOME,new Param(DOCS_HOME,this.getDocsHome()));	
		return preambule;
	}

	/**
	 *	Do the xradar post processing and generate the 3 files xradar requires to build the website
	 *	<ol>
	 *		<li>masterreport</li>
	 *		<li>system-quality</li>
	 *		<li>radar-normalized</li>
	 *	</ol>
	 * @param inputXml
	 * @param releaseData
	 * @param radarConfig
	 * @return
	 * @throws XRadarException
	 */
	public MergeProduct executePostProcessing(String inputXml,ReleaseData releaseData, Param radarConfig) throws XRadarException
	{
		String currentTargetFile = masterReport;
		try
		{
			MergeProduct products = new MergeProduct();
			// post processmaster
			products.setMasterReport(processStep(		new InputSource( new StringReader (inputXml)),
					POSTPROCESS,
					createPostProcessPreambule(releaseData,radarConfig),
					masterReport));
			products.setMasterReportFilename(masterReport);
			FileUtils.checkFileNotEmpty(masterReport);
			// Normalize XML file
			products.setRadarNormalized(processStep(	new InputSource(new FileInputStream(masterReport)),
					NORMALIZE,
					createPostProcessPreambule(releaseData,radarConfig),
					normalizedReport));
			products.setRadarNormalizedFilename(normalizedReport);
			currentTargetFile = normalizedReport;
			FileUtils.checkFileNotEmpty(normalizedReport);
			Preambule preambule = createPostProcessPreambule(releaseData,radarConfig);
			// TODO: I don't really remember why this next part is necessary... 
			// May be it no longer required as we fix stuff about win32 path at some other place
			if (XRadarUtils.isRunningOnWindowsOS()) 
				preambule.getParams().put(XML_REPORT_DIRECTORY,new Param(XML_REPORT_DIRECTORY,SYSTEM_FILE_SEPARATOR + getDocsHome() + SYSTEM_FILE_SEPARATOR + XML_DIRECTORY + SYSTEM_FILE_SEPARATOR + QUALITY_DIRECTORY));
			else
				preambule.getParams().put(XML_REPORT_DIRECTORY,new Param(XML_REPORT_DIRECTORY,getDocsHome() + SYSTEM_FILE_SEPARATOR + XML_DIRECTORY + SYSTEM_FILE_SEPARATOR + QUALITY_DIRECTORY));
			// ... maybe it has become useless ?
			// Generate system quality
			products.setSystemQuality(	processStep(new InputSource(new FileInputStream(normalizedReport)),
					QUALITY,
					preambule,
					systemQuality));
			products.setSystemQuality(StreamUtils.fromFilenameToInputStream(systemQuality));
			products.setSystemQualityFilename(systemQuality);
			currentTargetFile = systemQuality;
			FileUtils.checkFileNotEmpty(systemQuality);
			return products;
		}
		catch ( FileNotFoundException e ) {
			throw new XRadarException("Post merge treatement ended up with file" + currentTargetFile +  " not created.",e);
		}
	}

	/**
	 * <p>This method copy all the statics resources needed by xradar website from the xradar package distribution
	 * 		to the website folder. Statics resources includes stuff like xradar logos,...</p>
	 * TODO: Regroup those two parameter in a dedicated POJO to get a better signature
	 * @param imagesTargetPrefix, indicate in which directory to copy the images resources.
	 * @param xmlDataPrefix, indicate where to copy xml date file.
	 * @throws XRadarException 
	 */
	public void copyStaticResourcesToWebSite(String imagesTargetPrefix,String xmlDataPrefix) throws XRadarException
	{
		logger.info("Copying statics resources to websites");
		this.copyImagesToWebsite(XRadarUtils.normalizeFileName(imagesTargetPrefix));
		this.copyXmlDataToWebsite(XRadarUtils.normalizeFileName(xmlDataPrefix));
	}

	private void copyXmlDataToWebsite(String xmlDataPrefix) throws XRadarException
	{
		logger.fine("Loading xml data");
		String resourcePath = DefaultReportValues.getString("radar.internal.static.data.static");
		String targetPath = DefaultReportValues.getString("radar.internal.static.data.static.target");
		logger.fine("Copying " + resourcePath + " to " + xmlDataPrefix + SYSTEM_FILE_SEPARATOR + targetPath );
		try
		{
			StreamUtils.copy(new URL(resourcePath).openStream(),new FileOutputStream(new File(xmlDataPrefix + targetPath)));
		}
		catch (MalformedURLException e)
		{
			throw new XRadarException("Copy of XML Data to Website failed",e);
		}
		catch (IOException e)
		{
			throw new XRadarException("Copy of XML Data to Website failed",e);
		}
		catch (Exception e)
		{
			throw new XRadarException("Copy of XML Data to Website failed",e);
		}
	}



	public void closeProducts(MergeProduct product) throws XRadarException {
		if (product != null ) {
			StreamUtils.closeStream(product.getMasterReport());
			StreamUtils.closeStream(product.getSystemQuality());
			StreamUtils.closeStream(product.getRadarNormalized());
		}
	}


	/*
	 * Commodity method that regroup a generic version of all the preparation works
	 * to do before executing one of post process task.
	 */
	protected InputStream processStep(InputSource input,String step,Preambule preambule,String targetFile) throws XRadarException
	{
		InputStream out = null;
		logger.fine("Processing input with xsl:" + step);
		File target = new File(targetFile);
		merger.setXmlResult(new StreamResult(target));
		merger.setPreambule(preambule);
		merger.merge(input,StreamUtils.retrieveFileContentAsStream(step));		
		try 
		{
			out = new FileInputStream(target);
		}
		catch (FileNotFoundException e) 
		{
			throw new XRadarException("Can't read file" + targetFile,e);
		}
		return out;
	}

	/**
	 * @return the normalizedReport
	 */
	public String getNormalizedReport() {
		return normalizedReport;
	}
}
