/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;

import org.sourceforge.xradar.XRadarException;
import org.sourceforge.xradar.statics.Report;

/**
 * <p>An utility class to regroup every common functions
 * about files handling.</p>
 * 
 * @author rpelisse, belaran@gmail.com
 *
 */
public final class FileUtils {

	// As we mostly used URLs, the file separtor should always be '/'
	private static final String FILE_SEPARATOR = "/"; 
	// Should not be instantiated
	private FileUtils() {}

	/**
	 * <p>A simple commidity function to erase a file.</p>
	 * @param product
	 * @return
	 */
	public static boolean deleteFile(String product) {
		File toDelete = new File(product);
		if ( toDelete.exists() && toDelete.canWrite() )
			return toDelete.delete();
		return false;
	}

	/**
	 * <p>A commidity method to check if we can create a file.</p>
	 * @param filename
	 * @return
	 */
	public static boolean canFileBeCreated(String filename) {
		File toCreate = new File(filename);
		if ( ! toCreate.exists()  ) {
			try {
				if ( toCreate.createNewFile() ) {
					FileUtils.deleteFile(filename);
					return true;
				}
			} catch (IOException e) {
				// well, we can't create it so..;
				return false;
			}
		}
		return false;
	}

	/**
	 * <p>A commodity method to check if a file exists.</p>
	 * @param fileName
	 * @return
	 */
	public static boolean exists(String filename) {
		File file = new File(filename);
		return file.exists();
	}

	/**
	 * <p>Create any parent directory necessary for the provided filename.</p>
	 * @param filename,
	 */
	public static boolean createParentDirectories(String filename) {
//		FIXME: this class should have a logger and log this kind of debug info:		
//		System.out.println(">>> index:" + filename.lastIndexOf(FileUtils.FILE_SEPARATOR) + "[separator:" + FileUtils.FILE_SEPARATOR + "]");
//		System.out.println(">>> called with" + filename);
		File dir = new File(filename.substring(0,filename.lastIndexOf(FILE_SEPARATOR)));
		if ( ! dir.exists() )
			return dir.mkdirs();
		return true;
	}

	/**
	 * <p>Add the systemSepartor to the end of the provided directory path.</p>
	 * @param
	 * @return
	 */
	public static String addSeparatorIfMissing(String directoryName) {
		if (! directoryName.endsWith(FILE_SEPARATOR))
			return directoryName + FILE_SEPARATOR;
		return directoryName;
	}

	/**
	 * Save the content of product into file named accordingly to the content
	 * of target.
	 *
	 * @param target
	 * @param product
	 * @throws StaticsException
	 */
	public static void saveOnFile(String target,InputStream product) throws XRadarException
	{
		try
		{
			File file = new File(target);
			if ( ! file.createNewFile() && ! file.exists())
				throw new XRadarException("Failed to create file " + target);

			// This ignores the xml encoding specified on the header.
			// FileWriter javadoc: "The constructors of this class assume that the default character encoding and the default byte-buffer size are acceptable"
			//			FileWriter fileWriter = new FileWriter(file);
			//			fileWriter.write(product);
			//			fileWriter.close();
			//			
			// workaround: Try to find the encoding. If it not found use by default UTF-8.
			String encoding = "UTF-8";
			//FIXME: Repair this hack	
			/*			int firstIndex = product.indexOf("<?xml version=\"1.0\" encoding=\"");
			if (firstIndex != -1) {
				firstIndex = "<?xml version=\"1.0\" encoding=\"".length();
			}
			int lastIndex = product.indexOf("\"?>");
			if (firstIndex != -1 & lastIndex > firstIndex) {
				encoding = product.substring(firstIndex, lastIndex);
			}

			logger.finest("encoding for file \"" + target + "\" is :" + encoding);
			 */					
			OutputStreamWriter writer = new OutputStreamWriter (new FileOutputStream (file), encoding);
			char cbuf[] = new char[10000]; // TODO: Is this value the more relevant ?
			InputStreamReader reader = new InputStreamReader(product);
			int nbCharRead = reader.read();
			do 
			{
				writer.write(cbuf);
				nbCharRead = reader.read();
			}
			while ( nbCharRead != -1 ) ;
			writer.flush();
			writer.close();

		}
		catch (FileNotFoundException e)
		{
			throw new XRadarException("Can't write file  " + target,e);
		}
		catch (IOException e)
		{
			throw new XRadarException("Can't write file " + target,e);
		}
	}

	/**
	 * <p>A dummy method, that most likely need improvement, that allow to check if an
	 * XML file is not empty. It basicly check the size, in octet, of the file and ensure that it does
	 * have at least more than 8 characters, which is rougly the size of a proper XML prologue.</p>
	 *  
	 * @param filename
	 * @throws XRadarException
	 */
	public static void checkFileNotEmpty(String filename) throws XRadarException {
		File resultFile = new File(filename);
		if ( ! resultFile.exists() || (resultFile.length() < 8 ) ) {
			throw new XRadarException("Filename " + resultFile + " is missing or incomplete.");
		}
	}

	/**
	 * Create the specified directory
	 * @param dirPath
	 * @throws XRadarException
	 */
	public static void createDir(String dirPath) throws XRadarException
	{
		File directory = new File(dirPath);
		// If directory does not exist, we create it
		if ( ! directory.exists() && ! directory.mkdirs() ) 
			throw new XRadarException(dirPath + " does not exist, attempted to create it but failed");
		else if ( ! directory.isDirectory() )
			throw new XRadarException(dirPath + " exist but is not a directory !");
		//TODO: Check for  write access ?
	}

}
