/**
 * BSD-style license; for more info see http://xradar.sourceforge.net/license.html
 */
package org.sourceforge.xradar.util;

import java.io.BufferedInputStream;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.transform.stream.StreamResult;

import org.sourceforge.xradar.XRadarException;

/**
 * <p>An utility class to regroup every common functions
 * about stream handling.</p>
 * 
 * @author rpelisse, belaran@gmail.com
 *
 */
//TODO: Maybe a good candidate to create java5 static imports
public final class StreamUtils {

	private static final int EOF = -1;
	// Should not be instantiated
	private StreamUtils() {}
	
	/**
	 * <p>Handy method to easily copy from one inputstream to a file
	 * <code>
	 * 		copy(new File("source.dat"), new File("destination.dat"));
	 * </code>
	 * </p>
	 * @param src, inputstream <u>to copy</u>.
	 * @param dst, outstream.
	 * @throws Exception
	 */
	public static void copy(InputStream in, OutputStream out) throws XRadarException
	{
		try {
			StreamUtils.copyStreamToStream(in, out);
		} catch (FileNotFoundException e) {
			new XRadarException("Failed to copy",e);
		} catch (Exception e) {
			new XRadarException("Failed to copy",e);
		}
	}

	/**
	 * <p>Commodity method to copy a file into an other file, based only with 
	 * their filename. Also, wraps any exception around an XRadarException.</p>
	 * 
	 * @param fileIn
	 * @param fileOut
	 * @throws XRadarException
	 */
	public static void copyFileToFile(String fileIn, String fileOut) throws XRadarException
	{
		
		try {
			StreamUtils.copy(new FileInputStream(fileIn), new FileOutputStream(fileOut));
		} catch (FileNotFoundException e) {
			new XRadarException("Failed to copy",e);
		} catch (Exception e) {
			new XRadarException("Failed to copy",e);
		}
	}
	
	/**
	 * <p>Commodity method to copy an inputStream into an Outputstream</p>
	 * @param in
	 * @param out
	 * @throws IOException
	 */
	public static void copyStreamToStream(InputStream in, OutputStream out) throws IOException 
	{
		// copying data, byte to byte.
		byte[] buf = new byte[1024];
		int len;
		while ( ( len = in.read(buf) ) > 0 ) //NOPMD
			out.write(buf, 0, len);
		// closing both resources
		in.close();
		out.close();
	} 
	
	/*
	 * Commodity method
	 */
	public static StreamResult createOuput(Writer writer) {
		return new StreamResult(new BufferedWriter(writer));
	}

	/**
	 * <p>Commodity method to construct a simple FileWriter with its filename.</p>
	 * 
	 * @param output
	 * @return
	 * @throws XRadarException
	 */
	public static Writer createFileWriter(String output) throws XRadarException {
		Writer writer = null;
		try {
			writer = new FileWriter(new File(output));
		} catch (IOException e) {
			throw new XRadarException("Can't create file " + output,e);
		}
		return writer;
	}
	
	/**
	 * Get an InputStream from a path filename.
	 * @throws XRadarException 
	 */
	public static InputStream retrieveFileContentAsStream(String filename) throws XRadarException {
		try
		{
			//FIXME: move to calling this following line to any calling code 
			//logger.log(Level.FINE,"Retrieving filename from URL:" + filename);
			return new BufferedInputStream(new URL(filename).openStream());
		}
		catch (MalformedURLException e) {
			throw new XRadarException("Error while retrieving filename from URL" + filename,e);
		}
		catch (IOException e) {
			throw new XRadarException("Error while retrieving filename from URL" + filename,e);
		}
	}

	/**
	 * Get an InputStream from a file, on the file system. This method does not
	 * use the XRadar URL resolver,  and will fail with URL.
	 *
	 * @param filename
	 * @return
	 * @throws XRadarException
	 */
	public static InputStream readFile(String filename) throws XRadarException
	{
		try
		{
			return new FileInputStream(new File(filename));
		}
		catch (FileNotFoundException e)
		{
			throw new XRadarException("Unable to read file : " + filename,e);
		}
	}
	
	/**
	 * <p>Utility method to create an InputStream, based on a filename.</p>
	 * @param filename
	 * @return
	 * @throws XRadarException
	 */
	public static InputStream fromFilenameToInputStream(String filename) throws XRadarException {
		try 
		{
			return new FileInputStream(filename);
		} 
		catch (FileNotFoundException e) 
		{
			new XRadarException("Can't convert file to stream",e);
		}
		return null;
	}
	
	public static OutputStream createOutputStream(String filename) throws XRadarException 
	{
		try {
			return new FileOutputStream(new File(filename));
		} catch (FileNotFoundException e) {
			throw new XRadarException("Can't create file " + filename,e);
		}
	}
	
	public static InputStream returnStream(InputStream stream,String filename) {
		if ( stream.markSupported() ) {
			stream.mark(1);
			try  {
				if ( stream.read() == EOF ) 
					stream = StreamUtils.rebuildFileInputStream(stream, filename);
				else
					stream.reset();
			}
			catch (IOException error ) { // this should never happens !
				stream = StreamUtils.rebuildFileInputStream(stream, filename);
			}
		}
		else
		{
			try {
				stream.available(); // throw an exception if stream invalid so ...
			} 
			catch (IOException e) { // ... we created again the stream 
				stream = StreamUtils.rebuildFileInputStream(stream, filename);
			} 
		}
		return stream;
	}
	
	/**
	 * <p>Rather weird method wrote in order to rebuild a stream that can't be
	 * "rewinded". Please do not use this method, it is a hack, doomed to 
	 * disappear at some point.</p>
	 * @param stream
	 * @param filename
	 * @return
	 */
	public static InputStream rebuildFileInputStream(InputStream stream,String filename) {
		try {
			stream = new FileInputStream(filename);
		} 
		catch (FileNotFoundException fileNotFoundException) {
			// well, this should never happens... :) 
			stream = null;
		}
		return stream;
	}
	
	public static void closeStream(InputStream stream) throws XRadarException {
		if ( stream != null ) {
			try {
				stream.close();
			} catch (IOException e) {
				new XRadarException(e);
			}
		}
	}
}
